@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2025 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
 
  *  Authors: Török Edvin, Kevin Lin
@@ -157,7 +157,6 @@ enum FunctionalityLevels {
     FUNC_LEVEL_0103_9    = 130, /**< LibClamAV release 0.103.9 */
     FUNC_LEVEL_0103_10   = 131, /**< LibClamAV release 0.103.10 */
     FUNC_LEVEL_0103_11   = 132, /**< LibClamAV release 0.103.11 */
-    FUNC_LEVEL_0103_12   = 133, /**< LibClamAV release 0.103.12 */
 
     FUNC_LEVEL_0104   = 140, /**< LibClamAV release 0.104.0 */
     FUNC_LEVEL_0104_1 = 141, /**< LibClamAV release 0.104.1 */
@@ -177,6 +176,8 @@ enum FunctionalityLevels {
     FUNC_LEVEL_1_0_5 = 165, /**< LibClamAV release 1.0.5 */
     FUNC_LEVEL_1_0_6 = 166, /**< LibClamAV release 1.0.6 */
     FUNC_LEVEL_1_0_7 = 167, /**< LibClamAV release 1.0.7 */
+    FUNC_LEVEL_1_0_8 = 168, /**< LibClamAV release 1.0.8 */
+    FUNC_LEVEL_1_0_9 = 169, /**< LibClamAV release 1.0.9 */
 
     FUNC_LEVEL_1_1   = 180, /**< LibClamAV release 1.1.0 */
     FUNC_LEVEL_1_1_1 = 181, /**< LibClamAV release 1.1.1 */
@@ -194,6 +195,11 @@ enum FunctionalityLevels {
 
     FUNC_LEVEL_1_4   = 210, /**< LibClamAV release 1.4.0 */
     FUNC_LEVEL_1_4_1 = 211, /**< LibClamAV release 1.4.1 */
+    FUNC_LEVEL_1_4_2 = 212, /**< LibClamAV release 1.4.2 */
+    FUNC_LEVEL_1_4_3 = 213, /**< LibClamAV release 1.4.3 */
+
+    FUNC_LEVEL_1_5   = 230, /**< LibClamAV release 1.5.0 */
+    FUNC_LEVEL_1_5_1 = 231, /**< LibClamAV release 1.5.1 */
 };
 
 /**
@@ -265,7 +271,8 @@ enum pdf_objflags {
     OBJ_FILTER_STANDARD, /* */
     OBJ_LAUNCHACTION,    /* */
     OBJ_PAGE,            /* */
-    OBJ_CONTENTS         /* */
+    OBJ_CONTENTS,        /* */
+    OBJ_URI              /* */
 };
 
 /**
