@@ -1,7 +1,7 @@
 /*
  *  HTML Entity & Encoding normalization.
  *
- *  Copyright (C) 2013-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2025 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -57,7 +57,7 @@
 typedef struct {
     enum encodings encoding;
     size_t size;
-} * iconv_t;
+}* iconv_t;
 #endif
 
 static unsigned char tohex[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
@@ -157,7 +157,7 @@ static iconv_t iconv_open(const char* tocode, const char* fromcode)
 {
     UNUSEDPARAM(tocode);
 
-    iconv_t iconv = cli_malloc(sizeof(*iconv));
+    iconv_t iconv = malloc(sizeof(*iconv));
     if (!iconv)
         return NULL;
 
@@ -495,7 +495,7 @@ static char* normalize_encoding(const unsigned char* enc)
         if (!encname_chars[enc[i]])
             return NULL;
     }
-    norm = cli_malloc(len + 1);
+    norm = cli_max_malloc(len + 1);
     if (!norm)
         return NULL;
     for (i = 0; i < len; i++)
@@ -526,9 +526,11 @@ static void iconv_cache_init(struct iconv_cache* cache)
 static void iconv_cache_destroy(struct iconv_cache* cache)
 {
     size_t i;
-    cli_dbgmsg(MODULE_NAME "Destroying iconv pool:%p\n", (void*)cache);
+    // Don't use cli_dbgmsg() in destroy, because this happens *after* main() exits and we've already closed the log file handle.
+    //printf(MODULE_NAME "Destroying iconv pool:%p\n", (void*)cache);
     for (i = 0; i < cache->last; i++) {
-        cli_dbgmsg(MODULE_NAME "closing iconv:%p\n", cache->tab[i]);
+        // Don't log on destroy, because this happens *after* main() exits and we've already closed the log file handle.
+        //printf(MODULE_NAME "closing iconv:%p\n", cache->tab[i]);
         iconv_close(cache->tab[i]);
     }
     cli_hashtab_clear(&cache->hashtab);
@@ -584,7 +586,7 @@ static inline struct iconv_cache* cache_get_tls_instance(void)
 {
     struct iconv_cache* cache = pthread_getspecific(iconv_pool_tls_key);
     if (!cache) {
-        cache = cli_calloc(1, sizeof(*cache));
+        cache = calloc(1, sizeof(*cache));
         if (!cache) {
             cli_dbgmsg(MODULE_NAME "!Out of memory allocating TLS iconv instance\n");
             return NULL;
@@ -608,7 +610,7 @@ static void iconv_cache_cleanup_main(void)
 static inline void init_iconv_pool_ifneeded()
 {
     if (!iconv_global_inited) {
-        global_iconv_cache = cli_calloc(1, sizeof(*global_iconv_cache));
+        global_iconv_cache = calloc(1, sizeof(*global_iconv_cache));
         if (global_iconv_cache) {
             iconv_cache_init(global_iconv_cache);
             atexit(iconv_cache_cleanup_main);
@@ -656,7 +658,7 @@ static iconv_t iconv_open_cached(const char* fromcode)
         idx = cache->last++;
         if (idx >= cache->len) {
             cache->len += 16;
-            cache->tab = cli_realloc2(cache->tab, cache->len * sizeof(cache->tab[0]));
+            cache->tab = cli_max_realloc_or_free(cache->tab, cache->len * sizeof(cache->tab[0]));
             if (!cache->tab) {
                 cli_dbgmsg(MODULE_NAME "!Out of mem in iconv-pool\n");
                 errno = ENOMEM;
@@ -688,17 +690,31 @@ static int in_iconv_u16(const m_area_t* in_m_area, iconv_t* iconv_struct, m_area
         return 0;
     }
     /* convert encoding conv->tmp_area. conv->out_area */
-    alignfix = inleft % 4; /* iconv gives an error if we give him 3 bytes to convert,
-                               and we are using ucs4, ditto for utf16, and 1 byte*/
+
+    /*
+     * iconv gives an error if we give it less than 4 bytes to convert,
+     * and we are using ucs4, ditto for utf16, and 1 byte.
+     *
+     * If an alignfix is needed, we just trim the extra un-aligned
+     * bytes from the buffer.
+     * We hold on to the extra bytes, putting them into an aligned 4-byte
+     * buffer (tmp4), and then convert them with a final call to iconv.
+     */
+    alignfix = inleft % 4;
     inleft -= alignfix;
 
-    if (!inleft && alignfix) {
-        /* EOF, and we have less than 4 bytes to convert */
+    if (alignfix) {
+        /* Number of bytes is not mod(4).
+         * Copy the unaligned bytes from the end of the input.*/
         memset(tmp4, 0, 4);
-        memcpy(tmp4, input, alignfix);
-        input    = tmp4;
-        inleft   = 4;
-        alignfix = 0;
+        memcpy(tmp4, input + inleft, alignfix);
+
+        if (inleft == 0) {
+            /* Total number of bytes was < 4, so we only have the "unaligned" bytes to convert. */
+            inleft   = 4;
+            input    = tmp4;
+            alignfix = 0;
+        }
     }
 
     while (inleft && (outleft >= 2)) { /* iconv doesn't like inleft to be 0 */
@@ -711,6 +727,18 @@ static int in_iconv_u16(const m_area_t* in_m_area, iconv_t* iconv_struct, m_area
                 break;
             }
             /*cli_dbgmsg(MODULE_NAME "iconv error:%s\n", cli_strerror(errno, err, sizeof(err)));*/
+        } else if (0 == inleft) {
+            cli_dbgmsg(MODULE_NAME "iconv consumed all input\n");
+            if (alignfix) {
+                /* Convert the "unaligned" bytes. */
+                inleft   = 4;
+                input    = tmp4;
+                alignfix = 0;
+                continue;
+            } else {
+                /* no more data */
+                break;
+            }
         } else if (outleft == outleft_last) {
             cli_dbgmsg(MODULE_NAME "iconv stall (no output)\n");
         } else {
@@ -726,8 +754,21 @@ static int in_iconv_u16(const m_area_t* in_m_area, iconv_t* iconv_struct, m_area
         *out++ = 0;
         *out++ = *input++;
         inleft--;
+
+        if (0 == inleft && alignfix) {
+            /* Convert the "unaligned" bytes. */
+            inleft   = 4;
+            input    = tmp4;
+            alignfix = 0;
+            continue;
+        } else {
+            /* no more data */
+            break;
+        }
     }
+
     cli_dbgmsg("in_iconv_u16: unprocessed bytes: %lu\n", (unsigned long)inleft);
+
     if (out_m_area->length >= 0 && out_m_area->length >= (off_t)outleft) {
         out_m_area->length -= (off_t)outleft;
     } else {
@@ -803,7 +844,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
             int byte_count, sigbit_count;
 
             out_utf8_size = in_size;
-            out_utf8      = cli_calloc(1, out_utf8_size + 1);
+            out_utf8      = cli_max_calloc(1, out_utf8_size + 1);
             if (NULL == out_utf8) {
                 cli_errmsg("cli_codepage_to_utf8: Failure allocating buffer for utf8 filename.\n");
                 status = CL_EMEM;
@@ -863,7 +904,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                     uint16_t* pCodeUnits = (uint16_t*)in;
                     cchWideChar          = (int)in_size / 2;
 
-                    lpWideCharStr = cli_malloc((cchWideChar) * sizeof(WCHAR)); /* No need for a null terminator here, we'll deal with the exact size */
+                    lpWideCharStr = cli_max_malloc((cchWideChar) * sizeof(WCHAR)); /* No need for a null terminator here, we'll deal with the exact size */
                     if (NULL == lpWideCharStr) {
                         cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char string.\n");
                         status = CL_EMEM;
@@ -892,7 +933,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                         goto done;
                     }
 
-                    lpWideCharStr = cli_malloc((cchWideChar) * sizeof(WCHAR)); /* No need for a null terminator here, we'll deal with the exact size */
+                    lpWideCharStr = cli_max_malloc((cchWideChar) * sizeof(WCHAR)); /* No need for a null terminator here, we'll deal with the exact size */
                     if (NULL == lpWideCharStr) {
                         cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char string.\n");
                         status = CL_EMEM;
@@ -935,7 +976,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                 goto done;
             }
 
-            out_utf8 = cli_malloc(out_utf8_size + 1); /* Add a null terminator to this string */
+            out_utf8 = cli_max_malloc(out_utf8_size + 1); /* Add a null terminator to this string */
             if (NULL == out_utf8) {
                 cli_dbgmsg("cli_codepage_to_utf8: failed to allocate memory for wide char to utf-8 string.\n");
                 status = CL_EMEM;
@@ -994,7 +1035,7 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
 
                 outbytesleft = out_utf8_size;
 
-                out_utf8 = cli_calloc(1, out_utf8_size + 1);
+                out_utf8 = cli_max_calloc(1, out_utf8_size + 1);
                 if (NULL == out_utf8) {
                     cli_errmsg("cli_codepage_to_utf8: Failure allocating buffer for utf8 data.\n");
                     status = CL_EMEM;
@@ -1037,9 +1078,9 @@ cl_error_t cli_codepage_to_utf8(char* in, size_t in_size, uint16_t codepage, cha
                 }
 
                 /* iconv succeeded, but probably didn't use the whole buffer. Free up the extra memory. */
-                out_utf8_tmp = cli_realloc(out_utf8, out_utf8_size - outbytesleft + 1);
+                out_utf8_tmp = cli_max_realloc(out_utf8, out_utf8_size - outbytesleft + 1);
                 if (NULL == out_utf8_tmp) {
-                    cli_errmsg("cli_codepage_to_utf8: failure cli_realloc'ing converted filename.\n");
+                    cli_errmsg("cli_codepage_to_utf8: failure cli_max_realloc'ing converted filename.\n");
                     status = CL_EMEM;
                     goto done;
                 }
@@ -1100,7 +1141,7 @@ char* cli_utf16toascii(const char* str, unsigned int length)
     if (length % 2)
         length--;
 
-    if (!(decoded = cli_calloc(length / 2 + 1, sizeof(char))))
+    if (!(decoded = cli_max_calloc(length / 2 + 1, sizeof(char))))
         return NULL;
 
     for (i = 0, j = 0; i < length; i += 2, j++) {
@@ -1121,13 +1162,13 @@ char* cli_utf16_to_utf8(const char* utf16, size_t length, encoding_t type)
     char* s2;
 
     if (length < 2)
-        return cli_strdup("");
+        return cli_safer_strdup("");
     if (length % 2) {
         cli_warnmsg("utf16 length is not multiple of two: %lu\n", (long)length);
         length--;
     }
 
-    s2 = cli_malloc(needed);
+    s2 = cli_max_malloc(needed);
     if (!s2)
         return NULL;
 
