@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2023 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2025 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -108,7 +108,7 @@ static off_t pread_cb(void *handle, void *buf, size_t count, off_t offset)
     return pread((int)(ssize_t)handle, buf, count, offset);
 }
 
-fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name)
+fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name, const char *path)
 {
     STATBUF st;
     fmap_t *m = NULL;
@@ -132,12 +132,20 @@ fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const cha
     m = cl_fmap_open_handle((void *)(ssize_t)fd, offset, len, pread_cb, 1);
     if (!m)
         return NULL;
-    m->mtime = st.st_mtime;
+    m->mtime = (uint64_t)st.st_mtime;
 
     if (NULL != name) {
-        m->name = cli_strdup(name);
+        m->name = cli_safer_strdup(name);
         if (NULL == m->name) {
-            funmap(m);
+            fmap_free(m);
+            return NULL;
+        }
+    }
+
+    if (NULL != path) {
+        m->path = cli_safer_strdup(path);
+        if (NULL == m->path) {
+            fmap_free(m);
             return NULL;
         }
     }
@@ -152,25 +160,28 @@ static void unmap_win32(fmap_t *m)
         if (NULL != m->data) {
             UnmapViewOfFile(m->data);
         }
-        if (NULL != m->mh) {
-            CloseHandle(m->mh);
+        if (NULL != m->windows_map_handle) {
+            CloseHandle(m->windows_map_handle);
         }
         if (NULL != m->name) {
             free(m->name);
         }
+        if (NULL != m->path) {
+            free(m->path);
+        }
         free((void *)m);
     }
 }
 
-fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name)
+fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name, const char *path)
 { /* WIN32 */
     uint64_t pages, mapsz;
     int pgsz = cli_getpagesize();
     STATBUF st;
     fmap_t *m = NULL;
     const void *data;
-    HANDLE fh;
-    HANDLE mh;
+    HANDLE windows_file_handle;
+    HANDLE windows_map_handle;
 
     *empty = 0;
     if (FSTAT(fd, &st)) {
@@ -194,35 +205,43 @@ fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const cha
 
     pages = fmap_align_items(len, pgsz);
 
-    if ((fh = (HANDLE)_get_osfhandle(fd)) == INVALID_HANDLE_VALUE) {
+    if ((windows_file_handle = (HANDLE)_get_osfhandle(fd)) == INVALID_HANDLE_VALUE) {
         cli_errmsg("fmap: cannot get a valid handle for descriptor %d\n", fd);
         return NULL;
     }
-    if (!(mh = CreateFileMapping(fh, NULL, PAGE_READONLY, (DWORD)((len >> 31) >> 1), (DWORD)len, NULL))) {
+    if (!(windows_map_handle = CreateFileMapping(windows_file_handle, NULL, PAGE_READONLY, (DWORD)((len >> 31) >> 1), (DWORD)len, NULL))) {
         cli_errmsg("fmap: cannot create a map of descriptor %d\n", fd);
         return NULL;
     }
-    if (!(data = MapViewOfFile(mh, FILE_MAP_READ, (DWORD)((offset >> 31) >> 1), (DWORD)(offset), len))) {
+    if (!(data = MapViewOfFile(windows_map_handle, FILE_MAP_READ, (DWORD)((offset >> 31) >> 1), (DWORD)(offset), len))) {
         cli_errmsg("fmap: cannot map file descriptor %d\n", fd);
-        CloseHandle(mh);
+        CloseHandle(windows_map_handle);
         return NULL;
     }
     if (!(m = cl_fmap_open_memory(data, len))) {
         cli_errmsg("fmap: cannot allocate fmap_t\n", fd);
         UnmapViewOfFile(data);
-        CloseHandle(mh);
+        CloseHandle(windows_map_handle);
         return NULL;
     }
-    m->handle       = (void *)(size_t)fd;
-    m->handle_is_fd = 1; /* This is probably(?) needed so `fmap_fd()` can return the file descriptor. */
-    m->fh           = fh;
-    m->mh           = mh;
-    m->unmap        = unmap_win32;
+    m->handle              = (void *)(size_t)fd;
+    m->handle_is_fd        = true;
+    m->windows_file_handle = (void *)windows_file_handle;
+    m->windows_map_handle  = (void *)windows_map_handle;
+    m->unmap               = unmap_win32;
 
     if (NULL != name) {
-        m->name = cli_strdup(name);
+        m->name = cli_safer_strdup(name);
         if (NULL == m->name) {
-            funmap(m);
+            fmap_free(m);
+            return NULL;
+        }
+    }
+
+    if (NULL != path) {
+        m->path = cli_safer_strdup(path);
+        if (NULL == m->path) {
+            fmap_free(m);
             return NULL;
         }
     }
@@ -243,7 +262,7 @@ fmap_t *fmap_duplicate(cl_fmap_t *map, size_t offset, size_t length, const char
         goto done;
     }
 
-    duplicate_map = cli_malloc(sizeof(cl_fmap_t));
+    duplicate_map = malloc(sizeof(cl_fmap_t));
     if (!duplicate_map) {
         cli_warnmsg("fmap_duplicate: map allocation failed\n");
         goto done;
@@ -251,6 +270,9 @@ fmap_t *fmap_duplicate(cl_fmap_t *map, size_t offset, size_t length, const char
 
     /* Duplicate the state of the original map */
     memcpy(duplicate_map, map, sizeof(cl_fmap_t));
+    /* Clear the pointers that need to be unique pointers. */
+    duplicate_map->name = NULL;
+    duplicate_map->path = NULL;
 
     if (offset > map->len) {
         /* invalid offset, exceeds length of map */
@@ -287,13 +309,12 @@ fmap_t *fmap_duplicate(cl_fmap_t *map, size_t offset, size_t length, const char
         /* This also means the hash will be different.
          * Clear the have_<hash> flags.
          * It will be calculated when next it is needed. */
-        duplicate_map->have_md5    = false;
-        duplicate_map->have_sha1   = false;
-        duplicate_map->have_sha256 = false;
+        memset(duplicate_map->will_need_hash, 0, sizeof(duplicate_map->will_need_hash));
+        memset(duplicate_map->have_hash, 0, sizeof(duplicate_map->have_hash));
     }
 
     if (NULL != name) {
-        duplicate_map->name = cli_strdup(name);
+        duplicate_map->name = cli_safer_strdup(name);
         if (NULL == duplicate_map->name) {
             status = CL_EMEM;
             goto done;
@@ -302,11 +323,30 @@ fmap_t *fmap_duplicate(cl_fmap_t *map, size_t offset, size_t length, const char
         duplicate_map->name = NULL;
     }
 
+    /* Duplicate the path if it exists */
+    if (NULL != map->path) {
+        duplicate_map->path = cli_safer_strdup(map->path);
+        if (NULL == duplicate_map->path) {
+            status = CL_EMEM;
+            goto done;
+        }
+    } else {
+        duplicate_map->path = NULL;
+    }
+
     status = CL_SUCCESS;
 
 done:
     if (CL_SUCCESS != status) {
         if (NULL != duplicate_map) {
+            if (NULL != duplicate_map->name) {
+                free(duplicate_map->name);
+                duplicate_map->name = NULL;
+            }
+            if (NULL != duplicate_map->path) {
+                free(duplicate_map->path);
+                duplicate_map->path = NULL;
+            }
             free(duplicate_map);
             duplicate_map = NULL;
         }
@@ -322,6 +362,10 @@ void free_duplicate_fmap(cl_fmap_t *map)
             free(map->name);
             map->name = NULL;
         }
+        if (NULL != map->path) {
+            free(map->path);
+            map->path = NULL;
+        }
         free(map);
     }
 }
@@ -344,12 +388,15 @@ static void unmap_handle(fmap_t *m)
         if (NULL != m->name) {
             free(m->name);
         }
+        if (NULL != m->path) {
+            free(m->path);
+        }
         free((void *)m);
     }
 }
 
-extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
-                                      clcb_pread pread_cb, int use_aging)
+cl_fmap_t *fmap_open_handle(void *handle, size_t offset, size_t len,
+                            clcb_pread pread_cb, int use_aging)
 {
     cl_error_t status = CL_EMEM;
     uint64_t pages;
@@ -375,13 +422,13 @@ extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
     bitmap_size = pages * sizeof(uint64_t);
     mapsz       = pages * pgsz;
 
-    m = cli_calloc(1, sizeof(fmap_t));
+    m = calloc(1, sizeof(fmap_t));
     if (!m) {
         cli_warnmsg("fmap: map header allocation failed\n");
         goto done;
     }
 
-    m->bitmap = cli_calloc(1, bitmap_size);
+    m->bitmap = cli_max_calloc(1, bitmap_size);
     if (!m->bitmap) {
         cli_warnmsg("fmap: map header allocation failed\n");
         goto done;
@@ -408,7 +455,7 @@ extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
     }
 #endif /* ANONYMOUS_MAP */
     if (!use_aging) {
-        m->data = (fmap_t *)cli_malloc(mapsz);
+        m->data = (fmap_t *)cli_max_malloc(mapsz);
     }
     if (!m->data) {
         cli_warnmsg("fmap: map allocation failed\n");
@@ -416,7 +463,7 @@ extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
     }
     m->handle          = handle;
     m->pread_cb        = pread_cb;
-    m->aging           = use_aging;
+    m->aging           = use_aging != 0 ? true : false;
     m->offset          = offset;
     m->nested_offset   = 0;
     m->len             = len; /* m->nested_offset + m->len = m->real_len */
@@ -430,10 +477,10 @@ extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
     m->need_offstr     = handle_need_offstr;
     m->gets            = handle_gets;
     m->unneed_off      = handle_unneed_off;
-    m->handle_is_fd    = 1;
-    m->have_md5        = false;
-    m->have_sha1       = false;
-    m->have_sha256     = false;
+    m->handle_is_fd    = true;
+
+    memset(m->will_need_hash, 0, sizeof(m->will_need_hash));
+    memset(m->have_hash, 0, sizeof(m->have_hash));
 
     status = CL_SUCCESS;
 
@@ -587,7 +634,7 @@ static int fmap_readpage(fmap_t *m, uint64_t first_page, uint64_t count, uint64_
                             cli_warnmsg("fmap_readpage: fstat failed: %s\n", errtxt);
                             return 1;
                         }
-                        if (m->mtime != st.st_mtime) {
+                        if (m->mtime != (uint64_t)st.st_mtime) {
                             cli_warnmsg("fmap_readpage: file changed as we read it\n");
                             return 1;
                         }
@@ -723,7 +770,7 @@ static void unmap_mmap(fmap_t *m)
     size_t len = m->pages * m->pgsz;
     fmap_lock;
     if (munmap((void *)m->data, len) == -1) /* munmap() failed */
-        cli_warnmsg("funmap: unable to unmap memory segment at address: %p with length: %zu\n", (void *)m->data, len);
+        cli_warnmsg("fmap_free: unable to unmap memory segment at address: %p with length: %zu\n", (void *)m->data, len);
     fmap_unlock;
 #else
     UNUSEDPARAM(m);
@@ -736,6 +783,9 @@ static void unmap_malloc(fmap_t *m)
         if (NULL != m->name) {
             free(m->name);
         }
+        if (NULL != m->path) {
+            free(m->path);
+        }
         free((void *)m);
     }
 }
@@ -844,7 +894,7 @@ fmap_t *fmap_open_memory(const void *start, size_t len, const char *name)
     cl_error_t status = CL_ERROR;
 
     int pgsz     = cli_getpagesize();
-    cl_fmap_t *m = cli_calloc(1, sizeof(*m));
+    cl_fmap_t *m = calloc(1, sizeof(*m));
     if (!m) {
         cli_warnmsg("fmap: map allocation failed\n");
         goto done;
@@ -862,7 +912,7 @@ fmap_t *fmap_open_memory(const void *start, size_t len, const char *name)
 
     if (NULL != name) {
         /* Copy the name, if one is given */
-        m->name = cli_strdup(name);
+        m->name = cli_safer_strdup(name);
         if (NULL == m->name) {
             cli_warnmsg("fmap: failed to duplicate map name\n");
             goto done;
@@ -885,11 +935,6 @@ done:
     return m;
 }
 
-extern cl_fmap_t *cl_fmap_open_memory(const void *start, size_t len)
-{
-    return (cl_fmap_t *)fmap_open_memory(start, len, NULL);
-}
-
 static const void *mem_need(fmap_t *m, size_t at, size_t len, int lock)
 {
     UNUSEDPARAM(lock);
@@ -951,10 +996,10 @@ static const void *mem_gets(fmap_t *m, char *dst, size_t *at, size_t max_len)
     return dst;
 }
 
-fmap_t *fmap(int fd, off_t offset, size_t len, const char *name)
+fmap_t *fmap_new(int fd, off_t offset, size_t len, const char *name, const char *path)
 {
     int unused;
-    return fmap_check_empty(fd, offset, len, &unused, name);
+    return fmap_check_empty(fd, offset, len, &unused, name, path);
 }
 
 static inline uint64_t fmap_align_items(uint64_t sz, uint64_t al)
@@ -995,10 +1040,10 @@ cl_error_t fmap_dump_to_file(fmap_t *map, const char *filepath, const char *tmpd
 
     /* Create a filename prefix that includes the original filename, if available */
     if (filepath != NULL) {
-        if (CL_SUCCESS != cli_basename(filepath, strlen(filepath), &filebase)) {
+        if (CL_SUCCESS != cli_basename(filepath, strlen(filepath), &filebase, true /* posix_support_backslash_pathsep */)) {
             cli_dbgmsg("fmap_dump_to_file: Unable to determine basename from filepath.\n");
         } else if ((start_offset != 0) && (end_offset != map->real_len)) {
-            /* If we're only dumping a portion of the file, inlcude the offsets in the prefix,...
+            /* If we're only dumping a portion of the file, include the offsets in the prefix,...
              * e.g. tmp filename will become something like:  filebase.500-1200.<randhex> */
             size_t prefix_len = strlen(filebase) + 1 + SIZE_T_CHARLEN + 1 + SIZE_T_CHARLEN + 1;
             prefix            = malloc(prefix_len);
@@ -1081,14 +1126,9 @@ int fmap_fd(fmap_t *m)
     return fd;
 }
 
-extern void cl_fmap_close(cl_fmap_t *map)
+cl_error_t fmap_set_hash(fmap_t *map, uint8_t *hash, cli_hash_type_t type)
 {
-    funmap(map);
-}
-
-cl_error_t fmap_set_hash(fmap_t *map, unsigned char *hash, cli_hash_type_t type)
-{
-    cl_error_t status = CL_SUCCESS;
+    cl_error_t status = CL_ERROR;
 
     if (NULL == map) {
         cli_errmsg("fmap_set_hash: Attempted to set hash for NULL fmap\n");
@@ -1101,25 +1141,55 @@ cl_error_t fmap_set_hash(fmap_t *map, unsigned char *hash, cli_hash_type_t type)
         goto done;
     }
 
-    switch (type) {
-        case CLI_HASH_MD5:
-            memcpy(map->md5, hash, CLI_HASHLEN_MD5);
-            map->have_md5 = true;
-            break;
-        case CLI_HASH_SHA1:
-            memcpy(map->sha1, hash, CLI_HASHLEN_SHA1);
-            map->have_sha1 = true;
-            break;
-        case CLI_HASH_SHA256:
-            memcpy(map->sha256, hash, CLI_HASHLEN_SHA256);
-            map->have_sha256 = true;
-            break;
-        default:
-            cli_errmsg("fmap_set_hash: Unsupported hash type %u\n", type);
-            status = CL_EARG;
-            goto done;
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("fmap_set_hash: Unsupported hash type %u\n", type);
+        status = CL_EARG;
+        goto done;
     }
 
+    memcpy(map->hash[type], hash, cli_hash_len(type));
+    map->have_hash[type] = true;
+    status               = CL_SUCCESS;
+
+    if (cli_debug_flag) {
+        // Convert the hash to a hex string for logging
+        char hash_string[CLI_HASHLEN_MAX * 2 + 1] = {0};
+        size_t hash_len                           = cli_hash_len(type);
+
+        // Convert hash to string.
+        size_t i;
+        for (i = 0; i < hash_len; i++) {
+            sprintf(hash_string + i * 2, "%02x", map->hash[type][i]);
+        }
+        hash_string[hash_len * 2] = 0;
+
+        cli_dbgmsg("fmap_set_hash: set %s hash: %s\n", cli_hash_name(type), hash_string);
+    }
+
+done:
+    return status;
+}
+
+cl_error_t fmap_will_need_hash_later(fmap_t *map, cli_hash_type_t type)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (NULL == map) {
+        cli_errmsg("fmap_will_need_hash_later: Attempted to set hash for NULL fmap\n");
+        status = CL_EARG;
+        goto done;
+    }
+
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("fmap_will_need_hash_later: Unsupported hash type %u\n", type);
+        status = CL_EARG;
+        goto done;
+    }
+
+    /* set flags */
+    map->will_need_hash[type] = true;
+    status                    = CL_SUCCESS;
+
 done:
     return status;
 }
@@ -1128,54 +1198,40 @@ cl_error_t fmap_get_hash(fmap_t *map, unsigned char **hash, cli_hash_type_t type
 {
     cl_error_t status = CL_ERROR;
     size_t todo, at = 0;
-    void *hashctx = NULL;
+    void *hashctx[CLI_HASH_AVAIL_TYPES] = {NULL};
+    cli_hash_type_t hash_type;
 
     todo = map->len;
 
-    switch (type) {
-        case CLI_HASH_MD5:
-            if (map->have_md5) {
-                goto complete;
-            }
-            break;
-        case CLI_HASH_SHA1:
-            if (map->have_sha1) {
-                goto complete;
-            }
-            break;
-        case CLI_HASH_SHA256:
-            if (map->have_sha256) {
-                goto complete;
-            }
-            break;
-        default:
-            cli_errmsg("fmap_get_hash: Unsupported hash type %u\n", type);
-            status = CL_EARG;
-            goto done;
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("fmap_get_hash: Unsupported hash type %u\n", type);
+        status = CL_EARG;
+        goto done;
     }
 
+    /* If we already have the hash, just return it */
+    if (map->have_hash[type]) {
+        goto complete;
+    }
+
+    map->will_need_hash[type] = true;
+
     /*
-     * Need to calculate the hash.
+     * Need to calculate the requested hash and maybe others, as well.
      */
 
-    switch (type) {
-        case CLI_HASH_MD5:
-            hashctx = cl_hash_init("md5");
-            break;
-        case CLI_HASH_SHA1:
-            hashctx = cl_hash_init("sha1");
-            break;
-        case CLI_HASH_SHA256:
-            hashctx = cl_hash_init("sha256");
-            break;
-        default:
-            cli_errmsg("fmap_get_hash: Unsupported hash type %u\n", type);
-            status = CL_EARG;
-            goto done;
-    }
-    if (!(hashctx)) {
-        cli_errmsg("fmap_get_hash: error initializing new md5 hash!\n");
-        goto done;
+    /* Initialize hash contexts for all needed hash types */
+    for (hash_type = CLI_HASH_MD5; hash_type < CLI_HASH_AVAIL_TYPES; hash_type++) {
+        if (map->will_need_hash[hash_type] && !map->have_hash[hash_type]) {
+            const char *hash_name = cli_hash_name(hash_type);
+
+            hashctx[hash_type] = cl_hash_init(hash_name);
+            if (NULL == hashctx[hash_type]) {
+                cli_errmsg("fmap_get_hash: error initializing %s hash context\n", hash_name);
+                status = CL_EARG;
+                goto done;
+            }
+        }
     }
 
     while (todo) {
@@ -1191,58 +1247,415 @@ cl_error_t fmap_get_hash(fmap_t *map, unsigned char **hash, cli_hash_type_t type
         todo -= readme;
         at += readme;
 
-        if (cl_update_hash(hashctx, (void *)buf, readme)) {
-            cli_errmsg("fmap_get_hash: error calculating hash!\n");
-            status = CL_EREAD;
-            goto done;
+        for (hash_type = CLI_HASH_MD5; hash_type < CLI_HASH_AVAIL_TYPES; hash_type++) {
+            if (map->will_need_hash[hash_type] && !map->have_hash[hash_type]) {
+                if (cl_update_hash(hashctx[hash_type], buf, readme)) {
+                    const char *hash_name = cli_hash_name(hash_type);
+                    cli_errmsg("fmap_get_hash: error calculating %s hash!\n", hash_name);
+                    status = CL_EREAD;
+                    goto done;
+                }
+            }
         }
     }
 
-    switch (type) {
-        case CLI_HASH_MD5:
-            cl_finish_hash(hashctx, map->md5);
-            map->have_md5 = true;
-            break;
-        case CLI_HASH_SHA1:
-            cl_finish_hash(hashctx, map->sha1);
-            map->have_sha1 = true;
-            break;
-        case CLI_HASH_SHA256:
-            cl_finish_hash(hashctx, map->sha256);
-            map->have_sha256 = true;
-            break;
-        default:
-            cli_errmsg("fmap_get_hash: Unsupported hash type %u\n", type);
-            status = CL_EARG;
-            goto done;
+    for (hash_type = CLI_HASH_MD5; hash_type < CLI_HASH_AVAIL_TYPES; hash_type++) {
+        if (map->will_need_hash[hash_type] && !map->have_hash[hash_type]) {
+            cl_finish_hash(hashctx[hash_type], map->hash[hash_type]);
+            map->have_hash[hash_type] = true;
+
+            /* hashctx is finished, don't need to destroy it later */
+            hashctx[hash_type] = NULL;
+
+            if (cli_debug_flag) {
+                // Convert the hash to a hex string for logging
+                char hash_string[CLI_HASHLEN_MAX * 2 + 1] = {0};
+                size_t hash_len                           = cli_hash_len(hash_type);
+
+                // Convert hash to string.
+                size_t i;
+                for (i = 0; i < hash_len; i++) {
+                    sprintf(hash_string + i * 2, "%02x", map->hash[hash_type][i]);
+                }
+                hash_string[hash_len * 2] = 0;
+
+                cli_dbgmsg("fmap_get_hash: calculated %s hash: %s\n", cli_hash_name(hash_type), hash_string);
+            }
+        }
     }
-    hashctx = NULL;
 
 complete:
 
-    switch (type) {
-        case CLI_HASH_MD5:
-            *hash = map->md5;
-            break;
-        case CLI_HASH_SHA1:
-            *hash = map->sha1;
-            break;
-        case CLI_HASH_SHA256:
-            *hash = map->sha256;
-            break;
-        default:
-            cli_errmsg("fmap_get_hash: Unsupported hash type %u\n", type);
-            status = CL_EARG;
-            goto done;
+    *hash  = map->hash[type];
+    status = CL_SUCCESS;
+
+done:
+
+    for (hash_type = CLI_HASH_MD5; hash_type < CLI_HASH_AVAIL_TYPES; hash_type++) {
+        if (NULL != hashctx[hash_type]) {
+            cl_hash_destroy(hashctx[hash_type]);
+        }
+    }
+
+    return status;
+}
+
+/*
+ * Public API functions.
+ */
+
+extern cl_fmap_t *cl_fmap_open_handle(
+    void *handle,
+    size_t offset,
+    size_t len,
+    clcb_pread pread_cb,
+    int use_aging)
+{
+    return fmap_open_handle(handle, offset, len, pread_cb, use_aging);
+}
+
+extern cl_fmap_t *cl_fmap_open_memory(const void *start, size_t len)
+{
+    return (cl_fmap_t *)fmap_open_memory(start, len, NULL);
+}
+
+extern cl_error_t cl_fmap_set_name(cl_fmap_t *map, const char *name)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (!map || !name) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    free(map->name);
+
+    map->name = cli_safer_strdup(name);
+    if (!map->name) {
+        status = CL_EMEM;
+        goto done;
     }
 
     status = CL_SUCCESS;
 
 done:
+    return status;
+}
 
-    if (NULL != hashctx) {
-        cl_hash_destroy(hashctx);
+extern cl_error_t cl_fmap_get_name(cl_fmap_t *map, const char **name_out)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (!map || !name_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    *name_out = map->name;
+    status    = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_set_path(cl_fmap_t *map, const char *path)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (!map || !path) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    free(map->path);
+
+    map->path = cli_safer_strdup(path);
+    if (!map->path) {
+        status = CL_EMEM;
+        goto done;
+    }
+
+    status = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_get_path(cl_fmap_t *map, const char **path_out, size_t *offset_out, size_t *len_out)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (!map || !path_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    *path_out = map->path;
+    if (NULL == *path_out) {
+        status = CL_EACCES;
+        goto done;
+    }
+
+    if (offset_out) {
+        *offset_out = map->offset;
+    }
+
+    if (len_out) {
+        *len_out = map->len;
+    }
+
+    status = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_get_fd(const cl_fmap_t *map, int *fd_out, size_t *offset_out, size_t *len_out)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (!map || !fd_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    *fd_out = fmap_fd((fmap_t *)map);
+    if (*fd_out == -1) {
+        status = CL_EACCES;
+        goto done;
+    }
+
+    if (offset_out) {
+        *offset_out = map->offset;
+    }
+
+    if (len_out) {
+        *len_out = map->len;
+    }
+
+    status = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_get_size(const cl_fmap_t *map, size_t *size_out)
+{
+    cl_error_t status = CL_ERROR;
+
+    if (!map || !size_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    *size_out = map->len;
+    status    = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_set_hash(const cl_fmap_t *map, const char *hash_alg, char hash)
+{
+    cl_error_t status = CL_ERROR;
+    cli_hash_type_t type;
+
+    if (!map || !hash_alg) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    status = cli_hash_type_from_name(hash_alg, &type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_set_hash: Unknown hash algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("cl_fmap_set_hash: Unsupported hash algorithm: %s\n", hash_alg);
+        status = CL_EARG;
+        goto done;
+    }
+
+    status = fmap_set_hash((fmap_t *)map, (uint8_t *)&hash, type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_set_hash: Failed to set hash for algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_have_hash(const cl_fmap_t *map, const char *hash_alg, bool *have_hash_out)
+{
+    cl_error_t status = CL_ERROR;
+    cli_hash_type_t type;
+
+    if (!map || !hash_alg || !have_hash_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    status = cli_hash_type_from_name(hash_alg, &type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_have_hash: Unknown hash algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("cl_fmap_have_hash: Unsupported hash algorithm: %s\n", hash_alg);
+        status = CL_EARG;
+        goto done;
+    }
+
+    *have_hash_out = map->have_hash[type];
+    status         = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_will_need_hash_later(const cl_fmap_t *map, const char *hash_alg)
+{
+    cl_error_t status = CL_ERROR;
+    cli_hash_type_t type;
+
+    if (!map || !hash_alg) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    status = cli_hash_type_from_name(hash_alg, &type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_will_need_hash_later: Unknown hash algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("cl_fmap_will_need_hash_later: Unsupported hash algorithm: %s\n", hash_alg);
+        status = CL_EARG;
+        goto done;
+    }
+
+    status = fmap_will_need_hash_later((fmap_t *)map, type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_will_need_hash_later: Failed to indicate need for hash algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+    status = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern cl_error_t cl_fmap_get_hash(const cl_fmap_t *map, const char *hash_alg, char **hash_out)
+{
+    cl_error_t status = CL_ERROR;
+    cli_hash_type_t type;
+    unsigned char *hash;
+    char *hash_string = NULL;
+    size_t hash_len;
+    size_t i;
+
+    if (!map || !hash_alg || !hash_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    status = cli_hash_type_from_name(hash_alg, &type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_get_hash: Unknown hash algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+    if (type >= CLI_HASH_AVAIL_TYPES) {
+        cli_errmsg("cl_fmap_get_hash: Unsupported hash algorithm: %s\n", hash_alg);
+        status = CL_EARG;
+        goto done;
+    }
+
+    status = fmap_get_hash((fmap_t *)map, &hash, type);
+    if (status != CL_SUCCESS) {
+        cli_errmsg("cl_fmap_get_hash: Failed to get hash for algorithm: %s\n", hash_alg);
+        goto done;
+    }
+
+    hash_len = cli_hash_len(type);
+
+    /* Convert hash to string */
+    hash_string = malloc(hash_len * 2 + 1);
+    if (!hash_string) {
+        cli_errmsg("cl_fmap_get_hash: Failed to allocate memory for hash string\n");
+        status = CL_EMEM;
+        goto done;
+    }
+
+    for (i = 0; i < hash_len; i++) {
+        sprintf(hash_string + i * 2, "%02x", hash[i]);
+    }
+    hash_string[hash_len * 2] = 0;
+
+    *hash_out   = hash_string;
+    hash_string = NULL; /* transfer ownership to *hash_out */
+
+    status = CL_SUCCESS;
+
+done:
+    if (NULL != hash_string) {
+        free(hash_string);
+        hash_string = NULL;
     }
 
     return status;
 }
+
+extern cl_error_t cl_fmap_get_data(const cl_fmap_t *map, size_t offset, size_t len, const uint8_t **data_out, size_t *data_len_out)
+{
+    cl_error_t status = CL_ERROR;
+    const uint8_t *data;
+
+    if (!map || !data_out) {
+        status = CL_ENULLARG;
+        goto done;
+    }
+
+    if (offset > map->len) {
+        cli_errmsg("cl_fmap_get_data: Offset %zu is beyond end of file (file length %zu)\n", offset, map->len);
+        status = CL_EARG;
+        goto done;
+    }
+
+    if (len == 0) {
+        // If len is 0, we want to read to the end of the file.
+        len = map->len - offset;
+    }
+
+    if (offset + len > map->len) {
+        // Adjust len to read only to the end of the file if they asked for too much.
+        len = map->len - offset;
+    }
+
+    data = fmap_need_off_once_len((fmap_t *)map, offset, len, &len);
+    if (!data) {
+        cli_errmsg("cl_fmap_get_data: Failed to get data from fmap\n");
+        status = CL_EREAD;
+        goto done;
+    }
+
+    *data_out = data;
+    if (data_len_out) {
+        *data_len_out = len;
+    }
+
+    status = CL_SUCCESS;
+
+done:
+    return status;
+}
+
+extern void cl_fmap_close(cl_fmap_t *map)
+{
+    fmap_free(map);
+}
