@@ -38,6 +38,7 @@
 #include "vba_extract.h"
 #include "ole2_extract.h"
 #include "readdb.h"
+#include "clamav_rust.h"
 
 // common
 #include "output.h"
@@ -55,139 +56,7 @@ typedef struct mac_token2_tag {
 
 } mac_token2_t;
 
-cli_ctx *convenience_ctx(int fd)
-{
-    cl_error_t status        = CL_EMEM;
-    cli_ctx *ctx             = NULL;
-    struct cl_engine *engine = NULL;
-    cl_fmap_t *new_map       = NULL;
-
-    /* build engine */
-    engine = cl_engine_new();
-    if (NULL == engine) {
-        printf("convenience_ctx: engine initialization failed\n");
-        goto done;
-    }
-
-    cl_engine_set_num(engine, CL_ENGINE_AC_ONLY, 1);
-
-    if (cli_initroots(engine, 0) != CL_SUCCESS) {
-        printf("convenience_ctx: cli_initroots() failed\n");
-        goto done;
-    }
-
-    if (cli_add_content_match_pattern(engine->root[0], "test", "deadbeef", 0, 0, 0, "*", 0, NULL, 0) != CL_SUCCESS) {
-        printf("convenience_ctx: Can't parse signature\n");
-        goto done;
-    }
-
-    if (CL_SUCCESS != cl_engine_compile(engine)) {
-        printf("convenience_ctx: failed to compile engine.");
-        goto done;
-    }
-
-    /* fake input fmap */
-    new_map = fmap(fd, 0, 0, NULL);
-    if (NULL == new_map) {
-        printf("convenience_ctx: fmap failed\n");
-        goto done;
-    }
-
-    /* prepare context */
-    ctx = cli_calloc(1, sizeof(cli_ctx));
-    if (!ctx) {
-        printf("convenience_ctx: ctx allocation failed\n");
-        goto done;
-    }
-
-    ctx->engine = (const struct cl_engine *)engine;
-
-    ctx->dconf = (struct cli_dconf *)engine->dconf;
-
-    ctx->recursion_stack_size = ctx->engine->max_recursion_level;
-    ctx->recursion_stack      = cli_calloc(sizeof(recursion_level_t), ctx->recursion_stack_size);
-    if (!ctx->recursion_stack) {
-        status = CL_EMEM;
-        goto done;
-    }
-
-    // ctx was calloc'd, so recursion_level starts at 0.
-    ctx->recursion_stack[ctx->recursion_level].fmap = new_map;
-    ctx->recursion_stack[ctx->recursion_level].type = CL_TYPE_ANY; // ANY for the top level, because we don't yet know the type.
-    ctx->recursion_stack[ctx->recursion_level].size = new_map->len;
-
-    ctx->fmap = ctx->recursion_stack[ctx->recursion_level].fmap;
-
-    ctx->options = cli_calloc(1, sizeof(struct cl_scan_options));
-    if (!ctx->options) {
-        printf("convenience_ctx: scan options allocation failed\n");
-        goto done;
-    }
-    ctx->options->general |= CL_SCAN_GENERAL_HEURISTICS;
-    ctx->options->parse = ~(0);
-
-    status = CL_SUCCESS;
-
-done:
-    if (CL_SUCCESS != status) {
-        if (NULL != new_map) {
-            funmap(new_map);
-        }
-        if (NULL != ctx) {
-            if (NULL != ctx->options) {
-                free(ctx->options);
-            }
-            if (NULL != ctx->recursion_stack) {
-                free(ctx->recursion_stack);
-            }
-            free(ctx);
-            ctx = NULL;
-        }
-        if (NULL != engine) {
-            cl_engine_free(engine);
-        }
-    }
-
-    return ctx;
-}
-
-void destroy_ctx(cli_ctx *ctx)
-{
-    if (NULL != ctx) {
-        if (NULL != ctx->recursion_stack) {
-            /* Clean up any fmaps */
-            while (ctx->recursion_level > 0) {
-                if (NULL != ctx->recursion_stack[ctx->recursion_level].fmap) {
-                    funmap(ctx->recursion_stack[ctx->recursion_level].fmap);
-                    ctx->recursion_stack[ctx->recursion_level].fmap = NULL;
-                }
-                ctx->recursion_level -= 1;
-            }
-            if (NULL != ctx->recursion_stack[0].fmap) {
-                funmap(ctx->recursion_stack[0].fmap);
-                ctx->recursion_stack[0].fmap = NULL;
-            }
-
-            free(ctx->recursion_stack);
-            ctx->recursion_stack = NULL;
-        }
-
-        if (NULL != ctx->engine) {
-            cl_engine_free((struct cl_engine *)ctx->engine);
-            ctx->engine = NULL;
-        }
-
-        if (NULL != ctx->options) {
-            free(ctx->options);
-            ctx->options = NULL;
-        }
-
-        free(ctx);
-    }
-}
-
-int sigtool_vba_scandir(const char *dirname, int hex_output, struct uniq *U);
-
+/*only called by wm_decode_macro*/
 static char *get_unicode_name(char *name, int size)
 {
     int i, j;
@@ -217,6 +86,7 @@ static char *get_unicode_name(char *name, int size)
     return newname;
 }
 
+/*only called by wm_decode_macro*/
 static void output_token(unsigned char token)
 {
     int i;
@@ -313,6 +183,7 @@ static void output_token(unsigned char token)
     return;
 }
 
+/*only called by wm_decode_macro*/
 static void output_token67(uint16_t token)
 {
     int i;
@@ -663,6 +534,7 @@ static void output_token67(uint16_t token)
     return;
 }
 
+/*only called by wm_decode_macro*/
 static void output_token73(uint16_t token)
 {
     int i;
@@ -914,6 +786,7 @@ static void output_token73(uint16_t token)
     return;
 }
 
+/*only called by wm_decode_macro*/
 static void print_hex_buff(unsigned char *start, unsigned char *end, int hex_output)
 {
     if (!hex_output) {
@@ -944,6 +817,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x65:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 print_hex_buff(line_start, buff + i + 2 + s_length, hex_output);
@@ -955,6 +829,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x69:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf(" %s", tmp_buff);
@@ -964,6 +839,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x6a:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf(" \"%s\"", tmp_buff);
@@ -973,6 +849,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x6b:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf(" '%s", tmp_buff);
@@ -982,6 +859,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x6d:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf(" %s", tmp_buff);
@@ -991,6 +869,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x70:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf("REM%s", tmp_buff);
@@ -1000,6 +879,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x76:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf(" .%s", tmp_buff);
@@ -1009,6 +889,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x77:
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc(s_length + 1);
+                if (!tmp_buff) return;
                 strncpy((char *)tmp_buff, (char *)(buff + i + 2), s_length);
                 tmp_buff[s_length] = '\0';
                 printf("%s", tmp_buff);
@@ -1018,6 +899,7 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
             case 0x79: /* unicode "string" */
                 w_length = (uint16_t)(buff[i + 2] << 8) + buff[i + 1];
                 tmp_buff = (unsigned char *)malloc((w_length * 2) + 1);
+                if (!tmp_buff) return;
                 memcpy(tmp_buff, buff + i + 3, w_length * 2);
                 tmp_name = (unsigned char *)get_unicode_name((char *)tmp_buff, w_length * 2);
                 free(tmp_buff);
@@ -1025,10 +907,10 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
                 free(tmp_name);
                 i += 3 + (w_length * 2);
                 break;
-
             case 0x7c: /* unicode 'string */
                 s_length = (uint8_t)buff[i + 1];
                 tmp_buff = (unsigned char *)malloc((s_length * 2) + 1);
+                if (!tmp_buff) return;
                 memcpy(tmp_buff, buff + i + 2, s_length * 2);
                 tmp_name = (unsigned char *)get_unicode_name((char *)tmp_buff, s_length * 2);
                 free(tmp_buff);
@@ -1036,7 +918,6 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
                 free(tmp_name);
                 i += 2 + (s_length * 2);
                 break;
-
             case 0x66:
                 int_val = (uint8_t)(buff[i + 2] << 8) + buff[i + 1];
                 print_hex_buff(line_start, buff + i + 3, hex_output);
@@ -1092,264 +973,3 @@ static void wm_decode_macro(unsigned char *buff, uint32_t len, int hex_output)
     }
     print_hex_buff(line_start, buff + i, hex_output);
 }
-
-static int sigtool_scandir(const char *dirname, int hex_output)
-{
-    int status = -1;
-    DIR *dd    = NULL;
-    struct dirent *dent;
-    STATBUF statbuf;
-    const char *tmpdir;
-    char *fname    = NULL;
-    char *dir      = NULL;
-    cl_error_t ret = CL_CLEAN;
-    int desc       = -1;
-    cli_ctx *ctx   = NULL;
-    int has_vba = 0, has_xlm = 0;
-
-    if ((dd = opendir(dirname)) != NULL) {
-        while ((dent = readdir(dd))) {
-            if (dent->d_ino) {
-                if (strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..")) {
-                    /* build the full name */
-                    fname = (char *)cli_calloc(strlen(dirname) + strlen(dent->d_name) + 2, sizeof(char));
-                    if (!fname) {
-                        status = -1;
-                        goto done;
-                    }
-
-                    sprintf(fname, "%s" PATHSEP "%s", dirname, dent->d_name);
-
-                    /* stat the file */
-                    if (LSTAT(fname, &statbuf) != -1) {
-                        if (S_ISDIR(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode)) {
-                            if (sigtool_scandir(fname, hex_output)) {
-                                status = CL_VIRUS;
-                                goto done;
-                            }
-                        } else {
-                            if (S_ISREG(statbuf.st_mode)) {
-                                struct uniq *files = NULL;
-                                tmpdir             = cli_gettmpdir();
-
-                                /* generate the temporary directory */
-                                dir = cli_gentemp(tmpdir);
-                                if (!dir) {
-                                    printf("cli_gentemp() failed\n");
-                                    status = -1;
-                                    goto done;
-                                }
-
-                                if (mkdir(dir, 0700)) {
-                                    printf("Can't create temporary directory %s\n", dir);
-                                    status = CL_ETMPDIR;
-                                    goto done;
-                                }
-
-                                if ((desc = open(fname, O_RDONLY | O_BINARY)) == -1) {
-                                    printf("Can't open file %s\n", fname);
-                                    status = 1;
-                                    goto done;
-                                }
-
-                                if (!(ctx = convenience_ctx(desc))) {
-                                    status = 1;
-                                    goto done;
-                                }
-                                if ((ret = cli_ole2_extract(dir, ctx, &files, &has_vba, &has_xlm, NULL))) {
-                                    printf("ERROR %s\n", cl_strerror(ret));
-
-                                    status = ret;
-                                    goto done;
-                                }
-
-                                if (has_vba && files)
-                                    sigtool_vba_scandir(dir, hex_output, files);
-
-                                // Cleanup
-                                if (desc >= 0) {
-                                    close(desc);
-                                    desc = -1;
-                                }
-
-                                destroy_ctx(ctx);
-                                ctx = NULL;
-
-                                cli_rmdirs(dir);
-                                free(dir);
-                                dir = NULL;
-                            }
-                        }
-                    }
-
-                    // Cleanup
-                    free(fname);
-                    fname = NULL;
-                }
-            }
-        }
-    } else {
-        logg(LOGG_ERROR, "Can't open directory %s.\n", dirname);
-        return CL_EOPEN;
-    }
-
-    status = 0;
-
-done:
-    if (desc >= 0) {
-        close(desc);
-    }
-    if (NULL != dd) {
-        closedir(dd);
-    }
-    if (NULL != ctx) {
-        destroy_ctx(ctx);
-    }
-    if (NULL != dir) {
-        cli_rmdirs(dir);
-        free(dir);
-    }
-    if (NULL != fname) {
-        free(fname);
-    }
-
-    return status;
-}
-
-int sigtool_vba_scandir(const char *dirname, int hex_output, struct uniq *U)
-{
-    cl_error_t status = CL_CLEAN;
-    cl_error_t ret;
-    int i, fd;
-    size_t data_len;
-    vba_project_t *vba_project = NULL;
-    DIR *dd;
-    struct dirent *dent;
-    STATBUF statbuf;
-    char *fullname, vbaname[1024], *hash;
-    unsigned char *data;
-    uint32_t hashcnt;
-    unsigned int j;
-
-    if (CL_SUCCESS != (ret = uniq_get(U, "_vba_project", 12, NULL, &hashcnt))) {
-        logg(LOGG_ERROR, "ScanDir -> uniq_get('_vba_project') failed.\n");
-        return ret;
-    }
-
-    while (hashcnt) {
-        if (!(vba_project = (vba_project_t *)cli_vba_readdir(dirname, U, hashcnt))) {
-            hashcnt--;
-            continue;
-        }
-
-        for (i = 0; i < vba_project->count; i++) {
-            for (j = 0; j < vba_project->colls[i]; j++) {
-                snprintf(vbaname, 1024, "%s" PATHSEP "%s_%u", vba_project->dir, vba_project->name[i], j);
-                vbaname[sizeof(vbaname) - 1] = '\0';
-
-                fd = open(vbaname, O_RDONLY | O_BINARY);
-                if (fd == -1) continue;
-                data = (unsigned char *)cli_vba_inflate(fd, vba_project->offset[i], &data_len);
-                close(fd);
-
-                if (data) {
-                    data           = (unsigned char *)realloc(data, data_len + 1);
-                    data[data_len] = '\0';
-                    printf("-------------- start of code ------------------\n%s\n-------------- end of code ------------------\n", data);
-                    free(data);
-                }
-            }
-        }
-
-        cli_free_vba_project(vba_project);
-        vba_project = NULL;
-
-        hashcnt--;
-    }
-
-    if (CL_SUCCESS != (ret = uniq_get(U, "powerpoint document", 19, &hash, &hashcnt))) {
-        logg(LOGG_ERROR, "ScanDir -> uniq_get('powerpoint document') failed.\n");
-        return ret;
-    }
-
-    while (hashcnt) {
-        snprintf(vbaname, 1024, "%s" PATHSEP "%s_%u", dirname, hash, hashcnt);
-        vbaname[sizeof(vbaname) - 1] = '\0';
-
-        fd = open(vbaname, O_RDONLY | O_BINARY);
-        if (fd == -1) {
-            hashcnt--;
-            continue;
-        }
-        if ((fullname = cli_ppt_vba_read(fd, NULL))) {
-            sigtool_scandir(fullname, hex_output);
-            cli_rmdirs(fullname);
-            free(fullname);
-        }
-        close(fd);
-        hashcnt--;
-    }
-
-    if (CL_SUCCESS != (ret = uniq_get(U, "worddocument", 12, &hash, &hashcnt))) {
-        logg(LOGG_ERROR, "ScanDir -> uniq_get('worddocument') failed.\n");
-        return ret;
-    }
-
-    while (hashcnt) {
-        snprintf(vbaname, sizeof(vbaname), "%s" PATHSEP "%s_%u", dirname, hash, hashcnt);
-        vbaname[sizeof(vbaname) - 1] = '\0';
-
-        fd = open(vbaname, O_RDONLY | O_BINARY);
-        if (fd == -1) {
-            hashcnt--;
-            continue;
-        }
-
-        if (!(vba_project = (vba_project_t *)cli_wm_readdir(fd))) {
-            close(fd);
-            hashcnt--;
-            continue;
-        }
-
-        for (i = 0; i < vba_project->count; i++) {
-            data_len = vba_project->length[i];
-            data     = (unsigned char *)cli_wm_decrypt_macro(fd, vba_project->offset[i], (uint32_t)data_len, vba_project->key[i]);
-            if (data) {
-                data           = (unsigned char *)realloc(data, data_len + 1);
-                data[data_len] = '\0';
-                printf("-------------- start of code ------------------\n%s\n-------------- end of code ------------------\n", data);
-                free(data);
-            }
-        }
-
-        close(fd);
-        cli_free_vba_project(vba_project);
-        vba_project = NULL;
-        hashcnt--;
-    }
-
-    if ((dd = opendir(dirname)) != NULL) {
-        while ((dent = readdir(dd))) {
-            if (dent->d_ino) {
-                if (strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..")) {
-                    /* build the full name */
-                    fullname = calloc(strlen(dirname) + strlen(dent->d_name) + 2, sizeof(char));
-                    sprintf(fullname, "%s" PATHSEP "%s", dirname, dent->d_name);
-
-                    /* stat the file */
-                    if (LSTAT(fullname, &statbuf) != -1) {
-                        if (S_ISDIR(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode))
-                            sigtool_vba_scandir(fullname, hex_output, U);
-                    }
-                    free(fullname);
-                }
-            }
-        }
-    } else {
-        logg(LOGG_ERROR, "ScanDir -> Can't open directory %s.\n", dirname);
-        return CL_EOPEN;
-    }
-
-    closedir(dd);
-    return status;
-}
