@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2023 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2025 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -125,8 +125,6 @@ static const struct ac_sigopts_testdata_s {
 
     {NULL, 0, NULL, NULL, ACPATT_OPTION_NOOPTS, NULL, CL_CLEAN}};
 
-#if HAVE_PCRE
-
 static const struct pcre_testdata_s {
     const char *data;
     const char *hexsig;
@@ -152,8 +150,6 @@ static const struct pcre_testdata_s {
 
     {NULL, NULL, NULL, ACPATT_OPTION_NOOPTS, NULL, CL_CLEAN}};
 
-#endif /* HAVE_PCRE */
-
 static cli_ctx ctx;
 static struct cl_scan_options options;
 
@@ -170,16 +166,14 @@ static void setup(void)
     memset(&options, 0, sizeof(struct cl_scan_options));
     ctx.options = &options;
 
-    ctx.evidence = evidence_new();
-
     ctx.engine = cl_engine_new();
     ck_assert_msg(!!ctx.engine, "cl_engine_new() failed");
 
     ctx.dconf = ctx.engine->dconf;
 
     ctx.recursion_stack_size = ctx.engine->max_recursion_level;
-    ctx.recursion_stack      = cli_calloc(sizeof(recursion_level_t), ctx.recursion_stack_size);
-    ck_assert_msg(!!ctx.recursion_stack, "cli_calloc() for recursion_stack failed");
+    ctx.recursion_stack      = calloc(sizeof(cli_scan_layer_t), ctx.recursion_stack_size);
+    ck_assert_msg(!!ctx.recursion_stack, "calloc() for recursion_stack failed");
 
     // ctx was memset, so recursion_level starts at 0.
     ctx.recursion_stack[ctx.recursion_level].fmap = &thefmap;
@@ -198,8 +192,10 @@ static void setup(void)
 static void teardown(void)
 {
     cl_engine_free((struct cl_engine *)ctx.engine);
+    if (ctx.recursion_stack[ctx.recursion_level].evidence) {
+        evidence_free(ctx.recursion_stack[ctx.recursion_level].evidence);
+    }
     free(ctx.recursion_stack);
-    evidence_free(ctx.evidence);
 }
 
 START_TEST(test_ac_scanbuff)
@@ -285,11 +281,13 @@ START_TEST(test_ac_scanbuff_allscan)
         // phishingScan() doesn't check the number of alerts. When using CL_SCAN_GENERAL_ALLMATCHES
         // or if using `CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE` and `cli_append_potentially_unwanted()`
         // we need to count the number of alerts manually to determine the verdict.
-        ck_assert_msg(0 < evidence_num_alerts(ctx.evidence), "cli_scan_buff() failed for %s", ac_testdata[i].virname);
+        ck_assert_msg(0 < evidence_num_alerts(ctx.this_layer_evidence), "cli_scan_buff() failed for %s", ac_testdata[i].virname);
         ck_assert_msg(!strncmp(virname, ac_testdata[i].virname, strlen(ac_testdata[i].virname)), "Dataset %u matched with %s", i, virname);
-        if (evidence_num_alerts(ctx.evidence) > 0) {
-            evidence_free(ctx.evidence);
-            ctx.evidence = evidence_new();
+        if (evidence_num_alerts(ctx.this_layer_evidence) > 0) {
+            // Reset evidence for the next scan.
+            evidence_free(ctx.recursion_stack[ctx.recursion_level].evidence);
+            ctx.recursion_stack[ctx.recursion_level].evidence = NULL;
+            ctx.this_layer_evidence                           = NULL;
         }
     }
 
@@ -383,14 +381,16 @@ START_TEST(test_ac_scanbuff_allscan_ex)
         // phishingScan() doesn't check the number of alerts. When using CL_SCAN_GENERAL_ALLMATCHES
         // or if using `CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE` and `cli_append_potentially_unwanted()`
         // we need to count the number of alerts manually to determine the verdict.
-        if (0 < evidence_num_alerts(ctx.evidence)) {
+        if (0 < evidence_num_alerts(ctx.this_layer_evidence)) {
             verdict = CL_VIRUS;
         }
 
         ck_assert_msg(verdict == ac_sigopts_testdata[i].expected_result, "[ac_ex] cli_ac_scanbuff() failed for %s (%d != %d)", ac_sigopts_testdata[i].virname, verdict, ac_sigopts_testdata[i].expected_result);
-        if (evidence_num_alerts(ctx.evidence) > 0) {
-            evidence_free(ctx.evidence);
-            ctx.evidence = evidence_new();
+        if (evidence_num_alerts(ctx.this_layer_evidence) > 0) {
+            // Reset evidence for the next scan.
+            evidence_free(ctx.recursion_stack[ctx.recursion_level].evidence);
+            ctx.recursion_stack[ctx.recursion_level].evidence = NULL;
+            ctx.this_layer_evidence                           = NULL;
         }
     }
 
@@ -421,7 +421,7 @@ START_TEST(test_bm_scanbuff)
     ck_assert_msg(ret == CL_SUCCESS, "cli_add_content_match_pattern failed");
 
     ctx.options->general &= ~CL_SCAN_GENERAL_ALLMATCHES; /* make sure all-match is disabled */
-    ret = cli_bm_scanbuff((const unsigned char *)"blah\xde\xad\xbe\xef", 12, &virname, NULL, root, 0, NULL, NULL, NULL);
+    ret = cli_bm_scanbuff((const unsigned char *)"blah\xde\xad\xbe\xef", 8, &virname, NULL, root, 0, NULL, NULL, NULL);
     ck_assert_msg(ret == CL_VIRUS, "cli_bm_scanbuff() failed");
     ck_assert_msg(!strncmp(virname, "Sig2", 4), "Incorrect signature matched in cli_bm_scanbuff()\n");
 }
@@ -450,14 +450,12 @@ START_TEST(test_bm_scanbuff_allscan)
     ck_assert_msg(ret == CL_SUCCESS, "cli_add_content_match_pattern failed");
 
     ctx.options->general |= CL_SCAN_GENERAL_ALLMATCHES; /* enable all-match */
-    ret = cli_bm_scanbuff((const unsigned char *)"blah\xde\xad\xbe\xef", 12, &virname, NULL, root, 0, NULL, NULL, NULL);
+    ret = cli_bm_scanbuff((const unsigned char *)"blah\xde\xad\xbe\xef", 8, &virname, NULL, root, 0, NULL, NULL, NULL);
     ck_assert_msg(ret == CL_VIRUS, "cli_bm_scanbuff() failed");
     ck_assert_msg(!strncmp(virname, "Sig2", 4), "Incorrect signature matched in cli_bm_scanbuff()\n");
 }
 END_TEST
 
-#if HAVE_PCRE
-
 START_TEST(test_pcre_scanbuff)
 {
     struct cli_ac_data mdata;
@@ -472,13 +470,10 @@ START_TEST(test_pcre_scanbuff)
 #ifdef USE_MPOOL
     root->mempool = mpool_create();
 #endif
-    ret = cli_pcre_init();
-    ck_assert_msg(ret == CL_SUCCESS, "[pcre] cli_pcre_init() failed");
-
     for (i = 0; pcre_testdata[i].data; i++) {
         hexlen = strlen(PCRE_BYPASS) + strlen(pcre_testdata[i].hexsig) + 1;
 
-        hexsig = cli_calloc(hexlen, sizeof(char));
+        hexsig = calloc(hexlen, sizeof(char));
         ck_assert_msg(hexsig != NULL, "[pcre] failed to prepend bypass (out-of-memory)");
 
         strncat(hexsig, PCRE_BYPASS, hexlen);
@@ -526,13 +521,11 @@ START_TEST(test_pcre_scanbuff_allscan)
 #ifdef USE_MPOOL
     root->mempool = mpool_create();
 #endif
-    ret = cli_pcre_init();
-    ck_assert_msg(ret == CL_SUCCESS, "[pcre] cli_pcre_init() failed");
 
     for (i = 0; pcre_testdata[i].data; i++) {
         hexlen = strlen(PCRE_BYPASS) + strlen(pcre_testdata[i].hexsig) + 1;
 
-        hexsig = cli_calloc(hexlen, sizeof(char));
+        hexsig = calloc(hexlen, sizeof(char));
         ck_assert_msg(hexsig != NULL, "[pcre] failed to prepend bypass (out-of-memory)");
 
         strncat(hexsig, PCRE_BYPASS, hexlen);
@@ -565,15 +558,17 @@ START_TEST(test_pcre_scanbuff_allscan)
         // cli_scan_buff() doesn't check the number of alerts. When using CL_SCAN_GENERAL_ALLMATCHES
         // or if using `CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE` and `cli_append_potentially_unwanted()`
         // we need to count the number of alerts manually to determine the verdict.
-        if (0 < evidence_num_alerts(ctx.evidence)) {
+        if (0 < evidence_num_alerts(ctx.this_layer_evidence)) {
             verdict = CL_VIRUS;
         }
 
         ck_assert_msg(verdict == pcre_testdata[i].expected_result, "[pcre] cli_scan_buff() failed for %s", pcre_testdata[i].virname);
         /* num_virus field add to test case struct */
-        if (evidence_num_alerts(ctx.evidence) > 0) {
-            evidence_free(ctx.evidence);
-            ctx.evidence = evidence_new();
+        if (evidence_num_alerts(ctx.this_layer_evidence) > 0) {
+            // Reset evidence for the next scan.
+            evidence_free(ctx.recursion_stack[ctx.recursion_level].evidence);
+            ctx.recursion_stack[ctx.recursion_level].evidence = NULL;
+            ctx.this_layer_evidence                           = NULL;
         }
     }
 
@@ -581,8 +576,6 @@ START_TEST(test_pcre_scanbuff_allscan)
 }
 END_TEST
 
-#endif /* HAVE_PCRE */
-
 Suite *test_matchers_suite(void)
 {
     Suite *s = suite_create("matchers");
@@ -593,14 +586,10 @@ Suite *test_matchers_suite(void)
     tcase_add_test(tc_matchers, test_ac_scanbuff);
     tcase_add_test(tc_matchers, test_ac_scanbuff_ex);
     tcase_add_test(tc_matchers, test_bm_scanbuff);
-#if HAVE_PCRE
     tcase_add_test(tc_matchers, test_pcre_scanbuff);
-#endif
     tcase_add_test(tc_matchers, test_ac_scanbuff_allscan);
     tcase_add_test(tc_matchers, test_ac_scanbuff_allscan_ex);
     tcase_add_test(tc_matchers, test_bm_scanbuff_allscan);
-#if HAVE_PCRE
     tcase_add_test(tc_matchers, test_pcre_scanbuff_allscan);
-#endif
     return s;
 }
